import { AxiosRequestData } from '@/data/axios_request'
import cache from '@/utils/cache'

/**
 * 代记账客户设备信息汇总
 * @param {String} entName 企业名称
 * @param {Number} pageNum 当前页码  默认为1
 * @param {Number} pageSize 每页数量  默认为10
 */
function djzEntHardwareStatus({ entName, pageNum = 1, pageSize = 10 }) {
    // const url = '/sys/rpt/status/findPage'
    const url = '/sys/rpt/findPage'
    let token = cache.getToken()
    let mobile = JSON.parse(cache.getSession('user')).account
    let args = {
        pageNum: pageNum,
        pageSize: pageSize,
        mobile: mobile,
    }
    if (entName != '') {
        args.entName = entName;
    }
    console.log('代记账客户设备信息汇总 参数：')
    console.log(args)
    return new Promise(function (resolve, reject) {
        AxiosRequestData(url, token, args, false, 2)
            .then(val => {
                console.log('代记账客户设备信息汇总 结果：')
                console.log(val)
                if (val.code === 0) {
                    resolve(val)
                } else {
                    reject(val.msg)
                }
            })
    })
}

/**
 * 代记账客户发票汇总
 * @param {String} entName 企业名称
 * @param {String} timeFlag 1:天  2：月  3：季  4：年   默认为1
 * @param {String} statisticsDate timeFlag：1   格式：2020-09-09;timeFlag：2   格式：2020-09;timeFlag：3   格式：20203;timeFlag：4   格式：2020
 * @param {Number} pageNum 当前页码  默认为1
 * @param {Number} pageSize 每页数量  默认为10
 */
function djzEntInvoiceSummary({ entName, timeFlag, statisticsDate, pageNum = 1, pageSize = 10 }) {
    const url = '/sys/rpt/statistic/findPage'
    let token = cache.getToken()
    let mobile = JSON.parse(cache.getSession('user')).account
    let args = {
        timeFlag: timeFlag,
        statisticsDate: statisticsDate,
        pageNum: pageNum,
        pageSize: pageSize,
        mobile: mobile,
    }
    if (entName != '' && entName != null) {
        args.entName = entName;
    }
    console.log('代记账客户发票汇总 参数：')
    console.log(args)
    return new Promise(function (resolve, reject) {
        AxiosRequestData(url, token, args, false, 2)
            .then(val => {
                console.log('代记账客户发票汇总 结果：')
                console.log(val)
                if (val.code === 0) {
                    resolve(val)
                } else {
                    reject(val.msg)
                }
            })
    })
}

export {
    djzEntHardwareStatus,
    djzEntInvoiceSummary
}
