/**
 * @file 企业政策相关 WEB API
 */

import { AxiosRequestData } from "@/data/axios_request";
import cache from "@/utils/cache";

/**
 * 获取企业使用政策信息。
 * @param {string} policyId 政策唯一标识。
 */
export function getEntPolicy(policyId) {
  const url = "/sys/entPolicy/find";
  let token = cache.getToken();
  const { entTaxId } = JSON.parse(cache.getSession("company"));
  const args = {
    entTaxId,
    policyId,
    reqChannel: 4,
  };

  return new Promise(function (resolve, reject) {
    AxiosRequestData(url, token, args, false, 2).then((val) => {
      if (val.code === 0) {
        resolve(val);
      } else {
        reject(val);
      }
    });
  });
}

/**
 * 获取企业使用政策信息。
 * @param {string} policyId 政策唯一标识。
 * @param {boolean} useFlag 是否使用政策标识，false 不使用，true 使用。
 * @param {boolean} tipFlag 是否提示标志，true 提示，false 不提示。
 */
export function saveEntPolicy(params = {}) {
  const url = "/sys/entPolicy/save";
  let token = cache.getToken();
  const { entTaxId, entName } = JSON.parse(cache.getSession("company"));
  params.entTaxId = entTaxId;
  params.entName = entName;
  params.reqChannel = 4;

  return new Promise(function (resolve, reject) {
    AxiosRequestData(url, token, params, false, 1).then((val) => {
      if (val.code === 0) {
        resolve(val);
      } else {
        reject(val);
      }
    });
  });
}
